//import antlr.collections.List;
import de.fhpotsdam.unfolding.mapdisplay.*;
import de.fhpotsdam.unfolding.utils.*;
import de.fhpotsdam.unfolding.marker.*;
import de.fhpotsdam.unfolding.tiles.*;
import de.fhpotsdam.unfolding.interactions.*;
import de.fhpotsdam.unfolding.ui.*;
import de.fhpotsdam.unfolding.*;
import de.fhpotsdam.unfolding.core.*;
import de.fhpotsdam.unfolding.mapdisplay.shaders.*;
import de.fhpotsdam.unfolding.data.*;
import de.fhpotsdam.unfolding.geo.*;
import de.fhpotsdam.unfolding.texture.*;
import de.fhpotsdam.unfolding.events.*;
import de.fhpotsdam.utils.*;
import de.fhpotsdam.unfolding.providers.*;

import java.util.List;

import controlP5.*;

ControlP5 cp5;

UnfoldingMap map;

PFont font, fontklein;

String anlageDataFile = "test2.csv";

ArrayList<AnlagenDaten> gesamtDaten = new ArrayList();
int maxAnzahl = 70;
int sliderDate = 1990;
int sliderCash = 1989;

boolean datechanged = true;


color windrandcolor = color(40, 155, 205, 200);
color lablecolor = color(240, 240, 240, 200); //0, 155, 205, 200
color textcolor = color(40, 155, 205, 255); //80, 120, 130, 200
color citycolor = color(250, 185, 115, 150);
color strokecolor = color(255, 200, 150, 255);
color backgroundcolor = color(80, 80, 80, 120);

color sliderbg = color(240, 240, 240, 200);
color sliderfull = color(40, 155, 205, 200);
color slideractive = color(40, 155, 205, 200);
color hovercolor = color(0, 190, 255, 200);
color colorTicks = color(240, 240, 240, 255);

color bordercolor = color(0, 0, 0, 0);
color countrycolor = color(40, 50, 50, 230);


void setup() {
  size(1024, 768, P2D);
  smooth();
  
  font = createFont("Ubuntu", 25);
  fontklein = createFont("Ubuntu", 14);
  

  //Setup GUI
  cp5 = new ControlP5(this);

  cp5.addSlider("sliderDate")
    .setPosition(15, height-25)
      .setSize(width-20, 15)
        .setRange(1990, 2014) // values can range from big to small as well
          .setValue(1990)
            .setNumberOfTickMarks(24)
              .showTickMarks(true)
                .snapToTickMarks(true)
                  .setColorTickMark(colorTicks)
                    .setColorForeground(sliderfull)
                      .setColorBackground(sliderbg)
                        .setColorActive(sliderfull)
                          .setHandleSize(60)
                            .setSliderMode(Slider.FLEXIBLE)
                              .setTriggerEvent(Slider.RELEASE)
                                .setLabelVisible(false); 

  // reposition the Label for controller 'slider'
  cp5.getController("sliderDate").getValueLabel().align(ControlP5.LEFT, ControlP5.TOP_OUTSIDE).setPaddingX(0);
  cp5.getController("sliderDate").getCaptionLabel().align(ControlP5.RIGHT, ControlP5.TOP_OUTSIDE).setPaddingX(0);


  // Load CSV data
  Table anlagenDataCSV = loadTable(anlageDataFile, "header, csv");
  for (TableRow anlagenReihe : anlagenDataCSV.rows ()) {
    // Create new empty object to store data
    AnlagenDaten anlagenDaten = new AnlagenDaten();

    // Read data from CSV
    anlagenDaten.jahr = anlagenReihe.getInt("Jahr");
    anlagenDaten.name = anlagenReihe.getString("Name");
    anlagenDaten.kwh = anlagenReihe.getInt("KWh");
    anlagenDaten.anzahl = anlagenReihe.getInt("Anzahl");
    anlagenDaten.typ = anlagenReihe.getString("Typ");
    anlagenDaten.ort = anlagenReihe.getString("Ort");
    anlagenDaten.gemeinde = anlagenReihe.getString("Gemeinde");
    float lat = anlagenReihe.getFloat("Latitude");
    float lng = anlagenReihe.getFloat("Longitude");

    anlagenDaten.location = new Location(lat, lng);

    // Add to list of all bike stations
    gesamtDaten.add(anlagenDaten);

    // Debug Info
    println("Added " + anlagenDaten.name + " with " + anlagenDaten.jahr + " Jahr.");
  }

  println("Loaded " + gesamtDaten.size());

  // Karten Setup

  map = new UnfoldingMap(this);
  //map = new UnfoldingMap(this, new OpenStreetMap.OpenStreetMapProvider());
  //map = new UnfoldingMap(this, new StamenMapProvider.Toner());
  //map.setTweening(true);

  //MapUtils.createDefaultEventDispatcher(this, map);


  //markerFactory.setPolygonClass(CountryMarker.class);

   MarkerFactory markerFactory = new MarkerFactory();
 
   //markerFactory.setLineClass(BorderMarker.class);
   

  markerFactory.setPointClass(StaedteMarker.class);
  List<Feature> staedte = GeoJSONReader.loadData(this, "kreisstaedte.geojson");
  List<Marker> staedteMarkers = markerFactory.createMarkers(staedte);
  
   
  List<Feature> sachsenanhalt = GeoJSONReader.loadData(this, "gemeinden.geojson");
  List<Marker> saMarkers = MapUtils.createSimpleMarkers(sachsenanhalt);
  
  map.addMarkers(saMarkers);
  
  map.addMarkers(staedteMarkers);
  
  //Windkraft Weltweit
    
  /*List<Feature> windkraft = GeoJSONReader.loadData(this, "test.geojson");
   List<Marker> windkraftMarkers = markerFactory.createMarkers(windkraft);
   markerFactory.setPointClass(WindMarker.class);
   map.addMarkers(windkraftMarkers);*/

}

void draw() {

  fill(cp5.isMouseOver(cp5.getController("sliderDate")) ? hovercolor:color(sliderbg));
  background(backgroundcolor);
  map.draw();
  
  noStroke();
  
  fill(backgroundcolor);
  rect(0, 0, width, height);
  
  fill(sliderbg);
  rect(15,25,245,80);
  textFont(font);
  fill(textcolor);
  text("Windkraftanlagen", 30, 60);
  text("Sachsen-Anhalt", 30, 90);
  
  fill(sliderbg);
  rect(width-80,height-80,75,40);
  textFont(font);
  fill(textcolor);
  text(sliderDate, width-70, height-50);
  
 
  //Location location = map.getLocation(mouseX, mouseY);

  //text(location.getLat() + ", " + location.getLon(), mouseX, mouseY);

  //Zoom Abhängige Größe
  //float s = map.getZoom();

  //Festlegung Sachsen-Anhalt
  Location magdeburgLocation = new Location(52.1f, 11.6f);
  map.zoomAndPanTo(magdeburgLocation, 8);
  float maxPanningDistance = 0; // in km
  map.setPanningRestriction(magdeburgLocation, maxPanningDistance);

  
  //if (datechanged) { 
    //println("ich Rechne");
    // Iterate over all 
    for (AnlagenDaten anlagenDaten : gesamtDaten) {
      // Convert geo locations to screen positions
      ScreenPosition pos = map.getScreenPosition(anlagenDaten.location);
      // Map number of free bikes to radius of circle
      if (anlagenDaten.jahr <= int(sliderDate)) {
        fill(windrandcolor);
        float s = map(anlagenDaten.anzahl, 0, maxAnzahl, 1, 40);
        ellipse(pos.x, pos.y, s, s);
        
      

      if (anlagenDaten.showLabel) {
        
        String platzhalter;
        
        textFont(fontklein);
        fill(lablecolor);
        if(textWidth(anlagenDaten.name) < textWidth(anlagenDaten.gemeinde)){
          platzhalter = anlagenDaten.gemeinde;
        }else{
          platzhalter = anlagenDaten.name;
        }
        rect(pos.x-(textWidth(anlagenDaten.name)/2)-10, pos.y-20, textWidth(platzhalter)+20, 60);
        fill(textcolor);
        text(anlagenDaten.name, pos.x - textWidth(anlagenDaten.name)/2, pos.y);
        text(anlagenDaten.kwh +" KW/h", pos.x - textWidth(anlagenDaten.name)/2, pos.y+16);
        text(anlagenDaten.gemeinde, pos.x - textWidth(anlagenDaten.name)/2, pos.y+30);
      }
     }
    }
 
  // datechanged = false;
 //}
  
}



void keyPressed() {
  if (key == ' ') {
    map.getDefaultMarkerManager().toggleDrawing();
  }
}

void mouseClicked() {
  // Simple way of displaying bike station names. Use markers for single station selection.
  for (AnlagenDaten anlagenDaten : gesamtDaten) {
    anlagenDaten.showLabel = false;
    ScreenPosition pos = map.getScreenPosition(anlagenDaten.location);
    if (dist(pos.x, pos.y, mouseX, mouseY) < 5) {
      anlagenDaten.showLabel = true;
    }
  }
}
void controlEvent(ControlEvent theEvent) {

  float cash = 0;

  if (theEvent.value() != cash) {

    datechanged = true;
    println(theEvent.controller().name()+" = " + theEvent.value());
    cash = theEvent.value();
  } else {
    datechanged = false;
  }
}
